unit base;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Process;
Const
  ret = #13;

  Programm_sec  = 'Programm';
  Port_entry    = 'Port';
  UseSSL_entry  = 'UseSSL';
  UseLog_entry  = 'UseLog';
  UseAuth_entry = 'UseAuth';
  debug_entry   = 'debug';
  LogPfad_entry = 'LogPfad';
  ResetTime_entry = 'ResetTime';


  MD5FileName    = 'sk.md5';
  CertFileName   = 'sk.cert';
  KeyFilename    = 'sk.key';
  Config_Pfad         = '/etc/linuxmuster/maus/schukodaemon.cfg';
  DefLog_Pfad         = '/var/log/linuxmuster/schukodaemon.log';
  Workstations_Pfad   = '/etc/linuxmuster/workstations';
  Printers_Pfad       = '/etc/linuxmuster/printers';
  Room_Defaults_Pfad  = '/etc/linuxmuster/room_defaults';
  IN_Script_Pfad      = '/usr/share/linuxmuster/scripts/internet_on_off.sh';
  IR_Script_Pfad      = '/usr/share/linuxmuster/scripts/intranet_on_off.sh';
  UF_Script_Pfad      = '/usr/share/linuxmuster/scripts/urlfilter_on_off.sh';
  Helperfunction_Pfad = '/usr/share/linuxmuster/scripts/helperfunctions.sh';
  DistConf_Pfad       = '/usr/share/linuxmuster/config/dist.conf';
  Session_Pfad        = '/var/lib/schulkonsole/';


procedure Split(Delimiter: Char; Str: string; ListOfStrings: TStrings) ;
Function  Befehl_exec(Befehl : String):String;
Function  GetRaum(IP:String):String;
Function  GetRaumStatus(IP:String):String;
Function  GetRaumIPs(IP:String):String;
Function  GetRaumPCs(IP:String):String;
Function  GetRaumPCInfos(IP:String):String;
Function  GetRaumDrucker(IP:String):String;
Function  GetRaumDruckerStatus(IP:String):String;
Function  GetUserLogins(IP:String):String;
Function  GetUsersAktRaum(IP:String):String;
Function  CheckTeacher(Teacher:String):Boolean;
Function  CheckTeacherPW(Teacher,PW:String):Boolean;
Function  CheckTeacherHost(Teacher,IP:String):Boolean;
Function  LoescheRoomSession(Raum:String):Boolean;

Var
    useLog   : Boolean;
    useSSL   : Boolean;
    UseAuth  : Integer;
    debug    : boolean;
    SKPort   : Integer;
    LogPfad  : string;
    ResetTime : Integer;
    LastIP,LastRaum : String;

implementation

procedure Split(Delimiter: Char; Str: string; ListOfStrings: TStrings) ;
begin
   ListOfStrings.Clear;
   ListOfStrings.StrictDelimiter:=true;
   ListOfStrings.Delimiter     := Delimiter;
   ListOfStrings.DelimitedText := Str;
end;



procedure WSplit(Str: string; ListOfStrings: TStrings);
begin
   ListOfStrings.Clear;
   ListOfStrings.StrictDelimiter:=false;
   ListOfStrings.Delimiter     := ' ';
   ListOfStrings.DelimitedText := Str;
end;




procedure LogMessage(S : String);
Var
  Datei : TextFile;
  t     : String;
Begin
  t := FormatDateTime('dd.mm.yyyy-hh:nn:ss',now);
  Try
    if FileExists(LogPfad) then
      Begin
        AssignFile(Datei,LogPfad);
        Append(Datei);
        Writeln(Datei,t+' '+S);
        CloseFile(Datei);
      end
    else
      Begin
        AssignFile(Datei,LogPfad);
        Rewrite(Datei);
        Writeln(Datei,t+' '+S);
        CloseFile(Datei);
      end;
  except
    on E: EInOutError do
      writeln('File handling error occurred. Details: ', E.Message);
  end;
end;

Function  CheckTeacher(Teacher:String):Boolean;
Var
  Befehl : String;
Begin
  Befehl := 'sh -c "source '+Helperfunction_Pfad+' && source '+DistConf_Pfad+' && check_teacher '+teacher+' && echo true"';
  if Befehl_exec(Befehl)='true' then Result := true
                                else Result := false;
end;

Function  CheckTeacherPW(Teacher,PW:String):Boolean;
Var
  Befehl,DN,gidNumber : String;
  Ausgabe   : TStringList;
  n         : integer;
Begin
  Ausgabe := TStringList.create;
  Befehl := 'ldapsearch -x uid='+Teacher+' dn gidnumber';
  Ausgabe.text := Befehl_exec(Befehl);
  DN :='';
  gidNumber:='';
  for n := 0 to Ausgabe.count-1 do
    Begin
      if pos('dn:',Ausgabe[n])=1 then dn := Ausgabe[n];
      if pos('gidNumber:',Ausgabe[n])=1 then gidNumber := Ausgabe[n];
    end;
  if pos('10000', gidNumber)=0 then
    Begin
      LogMessage('Authentifizierung fehlgeschlagen: '+Teacher+' ist kein Lehrer.');
      result := false;
      exit;
    end;
  dn := trim(Copy(dn,5,length(dn)-4));
  Befehl := 'ldapwhoami -v -x -D "'+dn+'" -w "'+PW+'"';
  Ausgabe.text := Befehl_exec(Befehl);
  if pos('Success (0)',Ausgabe.text)>0 then
    Begin
      LogMessage('Authentifizierung OK: '+Teacher);
      result := True;
    end
  else
    begin
      LogMessage('Authentifizierung fehlgeschlagen: Invalid credentials.');
      result := false;
    end;
end;

Function CheckTeacherHost(Teacher,IP:String):Boolean;
Var
  Befehl,DN,gidNumber : String;
  Ausgabe   : TStringList;
  n         : integer;
  AllePC, AlleLogin,Zeile : TStringList;
  PCName    : String;
Begin
  result := false;
  Ausgabe := TStringList.create;
  Befehl := 'ldapsearch -x uid='+Teacher+' dn gidnumber';
  Ausgabe.text := Befehl_exec(Befehl);
  DN :='';
  gidNumber:='';
  for n := 0 to Ausgabe.count-1 do
    Begin
      if pos('dn:',Ausgabe[n])=1 then dn := Ausgabe[n];
      if pos('gidNumber:',Ausgabe[n])=1 then gidNumber := Ausgabe[n];
    end;
  if pos('10000', gidNumber)=0 then
    Begin
      LogMessage('Authentifizierung fehlgeschlagen: '+Teacher+' ist kein Lehrer.');
      exit;
    end;
  AllePC    := TStringList.create;
  AlleLogin := TStringList.create;
  Zeile     := TStringList.create;
  wsplit(GetRaumPCInfos(IP),AllePC);
  wsplit(GetUserLogins(IP),AlleLogin);
  PCName := '';
  for n := 0 to AlleLogin.count-1 do
    Begin
      Zeile.commatext := AlleLogin[n];
      if Zeile[1]=Teacher then PCName := Zeile[0];
    end;
  for n := 0 to AllePC.count-1 do
    Begin
      Zeile.commatext := AllePC[n];
      if (Zeile[0]=PCName) and (Zeile[1]=IP) then result := True;
    end;
  if Result=True then
    Begin
      LogMessage('Authentifizierung per Host OK: '+Teacher+','+PCName);
    end
  else
    Begin
      LogMessage('Authentifizierung fehlgeschlagen: Kein PC mit Lehrer-Login ('+Teacher+','+IP+','+PCName+').');
    end;
  AllePC.free;
  AlleLogin.free;
  Zeile.free;
end;


Function Befehl_exec(Befehl : String):String;
Var
  PRC     : TProcess;
  Ausgabe,Fehler : TStringList;
Begin
 PRC      := TProcess.Create(nil);
 Ausgabe  := TStringList.create;
 Fehler   := TStringList.create;
 PRC.commandline := Befehl;
 PRC.Options := PRC.Options + [poWaitOnExit, poUsePipes];
 PRC.Execute;
 Ausgabe.LoadFromStream(PRC.Output);
 Fehler.LoadFromStream(PRC.StDerr);
 if debug then LogMessage(Befehl);
 if debug then LogMessage(Ausgabe.text);
 Result := Trim(Ausgabe.text);
 if trim(Fehler.text)<>'' then if debug then LogMessage('ERROR: '+Fehler.text);
 Ausgabe.free;
 Fehler.free;
 PRC.free;
end;

Function GetRaumIPs(IP:String):String;
Var
  Workstations, Zeile : TStringList;
  n                   : Integer;
  Raum,z,IPList : String;
Begin
  if FileExists(Workstations_Pfad) then
     Begin
       Raum := GetRaum(IP);
       IPList := '';
       Workstations := TStringList.create;
       Zeile        := TStringList.create;
       Try
         Workstations.LoadFromFile(Workstations_Pfad);
       except
         On E : Exception do LogMessage('ERROR: '+E.Message);
       end;
       for n := 0 to Workstations.Count-1 do
         Begin
           z := trim(Workstations[n]);
           if (length(z)>11) and (z[1]<>'#') then
             Begin
               Split(';',z,Zeile);
               if (Zeile.count>4) and (Zeile[0]=Raum) and (Zeile[4]<>IP) then
                 Begin
                   if IPList='' then IPList := Zeile[4]
                                else IPList := IPList + ','+Zeile[4];
                 end;
             end;
         end;
       Result := IPList;
       Zeile.free;
       Workstations.free;
    end
  else
    Begin
      LogMessage('ERROR: Die Datei '+Workstations_Pfad+' existiert nicht.');
      Result :='';
    end;
end;

Function GetRaumPCs(IP:String):String;
Var
  Workstations, Zeile : TStringList;
  n                   : Integer;
  Raum,z,PCList : String;
Begin
  if FileExists(Workstations_Pfad) then
     Begin
       Raum := GetRaum(IP);
       PCList := '';
       Workstations := TStringList.create;
       Zeile        := TStringList.create;
       Try
         Workstations.LoadFromFile(Workstations_Pfad);
       except
         On E : Exception do LogMessage('ERROR: '+E.Message);
       end;
       for n := 0 to Workstations.Count-1 do
         Begin
           z := trim(Workstations[n]);
           if (length(z)>11) and (z[1]<>'#') then
             Begin
               Split(';',z,Zeile);
               if (Zeile.count>4) and (Zeile[0]=Raum) then
                 Begin
                   if PCList='' then PCList := Zeile[1]
                                else PCList := PCList + ','+Zeile[1];
                 end;
             end;
         end;
       Result := PCList;
       Zeile.free;
       Workstations.free;
    end
  else
    Begin
      LogMessage('ERROR: Die Datei '+Workstations_Pfad+' existiert nicht.');
      Result :='';
    end;
end;

Function GetRaumPCInfos(IP:String):String;
Var
  Workstations, Zeile : TStringList;
  n                   : Integer;
  Raum,z,s,  PCList   : String;
Begin
  if FileExists(Workstations_Pfad) then
     Begin
       Raum := GetRaum(IP);
       PCList := '';
       Workstations := TStringList.create;
       Zeile        := TStringList.create;
       Try
         Workstations.LoadFromFile(Workstations_Pfad);
       except
         On E : Exception do LogMessage('ERROR: '+E.Message);
       end;
       for n := 0 to Workstations.Count-1 do
         Begin
           z := trim(Workstations[n]);
           if (length(z)>11) and (z[1]<>'#') then
             Begin
               Split(';',z,Zeile);
               if (Zeile.count>4) and (Zeile[0]=Raum) then
                 Begin
                   s := Zeile[1]+','+Zeile[4]+','+Zeile[3];
                   if PCList='' then PCList := s
                                else PCList := PCList + ' '+s;
                 end;
             end;
         end;
       Result := PCList;
       Zeile.free;
       Workstations.free;
    end
  else
    Begin
      LogMessage('ERROR: Die Datei '+Workstations_Pfad+' existiert nicht.');
      Result :='';
    end;
end;



Function GetRaum(IP:String):String;
Var
  Workstations, Zeile : TStringList;
  n                   : Integer;
  z                   : String;
Begin
  if (IP=LastIP) and (LastRaum<>'') then
    Begin
      Result :=LastRaum;
      exit;
    end;
  Result :='';
  if FileExists(Workstations_Pfad) then
     Begin
       Workstations := TStringList.create;
       Zeile        := TStringList.create;
       Try
         Workstations.LoadFromFile(Workstations_Pfad);
       except
         On E : Exception do LogMessage('ERROR: '+E.Message);
       end;
       for n := 0 to Workstations.Count-1 do
         Begin
           z := trim(Workstations[n]);
           if (length(z)>11) and  (z[1]<>'#') then
                 Begin
                   Split(';',z,Zeile);
                   if (Zeile.count>4) and (Trim(Zeile[4])=IP) then
                     Begin
                       Result := Trim(Zeile[0]);
                       LogMessage('Raum erkannt: '+IP+' -> '+Result);
                       LastIP:=IP;
                       LastRaum:=Result;
                     end;
                 end;
         end;
       Zeile.free;
       Workstations.free;
     end
  else
    Begin
      LogMessage('ERROR: Die Datei '+Workstations_Pfad+' existiert nicht.');
    end;
end;

Function GetRaumStatus(IP:String):String;
Var
  Raeume, Zeile : TStringList;
  Raum,z,r,d    : String;
  n             : Integer;
Begin
  Raum := GetRaum(IP);
  Result :='';
  if FileExists(Room_Defaults_Pfad) then
    Begin
      if debug then LogMessage('RoomDefaults_pfad erkannt');
      Raeume := TStringList.create;
      Zeile  := TStringList.create;
      Try
        Raeume.LoadFromFile(Room_Defaults_Pfad);
      except
        On E : Exception do LogMessage('ERROR: '+E.Message);
      end;
      d := '';
      r := '';
      for n := 0 to Raeume.count-1 do
        Begin
          z := trim(Raeume[n]);
          if (length(z)>1) and  (z[1]<>'#') then
            Begin
              wsplit(z,Zeile);
              if (Zeile.count =4) then
                Begin
                  If Zeile[0]=Raum then
                    Begin
                      r := Zeile[1]+','+Zeile[2]+','+Zeile[3];
                      break;
                    end
                  else if Zeile[0]='default' then
                    Begin
                      d := Zeile[1]+','+Zeile[2]+','+Zeile[3];
                    end;
                end;
            end;
        end;
      if r <>'' then result := r
                else result := d;
      Zeile.free;
      Raeume.free;
    end
  else
    Begin
      LogMessage('ERROR: Die Datei '+Room_Defaults_Pfad+' existiert nicht.');
    end;
end;

Function GetRaumDrucker(IP:String):String;
Var
  Printers, Zeile, Raeume : TStringList;
  Raum, z : String;
  n,m     : Integer;
Begin
  Result :='';
  Raum := GetRaum(IP);
  if FileExists(Printers_Pfad) then
    Begin
      if debug then LogMessage('Printers_pfad erkannt');
      Printers := TStringList.create;
      Zeile    := TStringList.create;
      Raeume   := TStringList.create;
      Try
        Printers.LoadFromFile(Printers_Pfad);
      except
        On E : Exception do LogMessage('ERROR: '+E.Message);
      end;
      for n := 0 to Printers.count-1 do
        Begin
          z := trim(Printers[n]);
          if (length(z)>1) and  (z[1]<>'#') then
            Begin
              WSplit(z,Zeile);
              if Zeile.count >1 then
                Begin
                  Raeume.commatext := Trim(Zeile[1]);
                  for m := 0 to Raeume.count-1 do
                    Begin
                      if Raeume[m]=Raum then
                        Begin
                          if Result = '' then Result := Zeile[0]
                                         else Result := Result +' '+Zeile[0];
                        end;
                    end;
                end;
            end;
        end;
      Printers.free;
      Zeile.free;
      Raeume.free;
      if Result ='' then LogMessage('Keine Drucker im Raum vorhanden.')
                    else LogMessage('Raumdrucker: '+Result);
    end
  else
    Begin
      LogMessage('ERROR: Die Datei '+Printers_Pfad+' existiert nicht.');
    end;
end;

Function GetRaumDruckerStatus(IP:String):String;
Const
  NAR = 'not accepting requests';
Var
  Drucker : TStringList;
  Befehl,Ausgabe : String;
Begin
  Drucker := TStringList.create;
  wSplit(GetRaumDrucker(IP), Drucker);
  If Drucker.count>0 then
    Begin
      Befehl := 'lpstat -a '+Drucker[0];
      Ausgabe := Befehl_exec(Befehl);
      if pos(NAR,lowercase(Ausgabe))>0 then result :='REJECT'
                                       else result :='ACCEPT';
    end
  else
    Begin
      Result := '';
    end;
  Drucker.free;
end;

Function  GetUserLogins(IP:String):String;
Var
  PCName, PCLogin, Zeile, Ausgabe : TStringList;
  Befehl, URL, PC : String;
  n, i, a : integer;
Begin
  Result  :='';
  PCName  := TStringList.create;
  PCLogin := TStringList.create;
  Zeile   := TStringList.create;
  Ausgabe := TStringList.create;
  PCName.commatext := GetRaumPCs(IP);
  if PCName.count>0 then
    Begin
      for i := 0 to (PCName.count-1) do PCLogin.Append('-');
      Befehl := 'smbstatus -b';
      Ausgabe.text := Befehl_exec(Befehl);
      for n := 0 to (Ausgabe.count-1) do
        Begin
          wsplit(Ausgabe[n],Zeile);
          if ((Zeile.count=4) or (Zeile.count=5) ) and (StrToIntDef(Zeile[0],0)>0) then
            Begin
              URL := Zeile[Zeile.count-1];
              a := Pos('.',URL);
              if (a>2) and (URL[1]='(') then
                Begin
                  PC := Copy(URL,2,a-2);
                  for i := 0 to PCName.count-1 do
                    Begin
                      if PC = PCName[i] then PCLogin[i] := Zeile[1];
                    end;
                end;
            end;

        end;
      for i := 0 to (PCName.count-1) do
        Begin
          if Result ='' then Result := PCName[i]+','+PCLogin[i]
                        else Result := Result + ' '+ PCName[i]+','+PCLogin[i];
        end;
    end;
  Zeile.free;
  PCName.free;
  PCLogin.free;
  Ausgabe.free;
end;

Function GetUsersAktRaum(IP:String):String;
Var
  Liste, Zeile : TStringList;
  n : integer;
Begin
  Liste := TStringList.create;
  Zeile := TStringList.create;
  WSplit(GetUserLogins(IP),Liste);
  Result :='';
  for n := 0 to Liste.count-1 do
    Begin
      Zeile.commatext := Liste[n];
      if (Zeile.count=2) and (Zeile[1]<>'-') then
        Begin
          if result='' then Result := Zeile[1]
                       else Result := Result+','+Zeile[1];
        end;
    end;
  Zeile.free;
  Liste.free;
end;

Function  LoescheRoomSession(Raum:String):Boolean;
Var
  Ausgabe, Datei : TStringList;
  Befehl  : String;
  n       : Integer;
Begin
  Result := false;
  if DirectoryExists(Session_Pfad) then
    Begin
      Ausgabe := TStringList.create;
      Datei   := TStringList.create;
      Befehl := 'sh -c "ls -1 '+Session_pfad+'room_*"';
      Ausgabe.text := Befehl_exec(Befehl);
      for n := 0 to Ausgabe.count-1 do
        Begin
          if FileExists(Ausgabe[n]) then
            Begin
              Try
                Datei.LoadFromFile(Ausgabe[n]);
              except
                On E : Exception do LogMessage('ERROR: '+E.Message);
              end;
              if pos('''name'' => '''+Raum+'''',Datei.text)>0 then
                Begin
                  if debug then LogMessage('Session-Datei '+Ausgabe[n]+' wird gelöscht ...');
                  Try
                    DeleteFile(Ausgabe[n]);
                  except
                    On E : Exception do LogMessage('ERROR: '+E.Message);
                  end;
                  result := True;
                end;
            end;
        end;
      Datei.free;
      Ausgabe.free;
    end
  else
    LogMessage('Das Verzeichnis '+Session_Pfad+' existiert nicht!');
end;

end.

