// schukodaemon fuer linuxmuster.net
//
// programmiert von Martin Resch
//
// 0.1 erste Version
// + mit Auth - nur Lehrer
// + logging ändern
// 0.12
// + printer on/off
// + intranet, URL-Filter on/off
// + Raum Status
// + Set Room Default
// 0.20
// + Get Logins
// + Get Room
// + Error Handling
// 0.22
// + room_session Datei löschen
// 0.23
// + BUG: bei Windows-share
// 0.25
// + Datum in Log-Datei
// + Auth hostbasiert
// + Raum Reset nach vorgegebener Zeit
// 0.26
// + etcpfad=/etc/linuxmuster/maus
// 0.27
// + kleine Anpassungen
// 0.28
// - WOL
// - useSSL
//
//

unit daemonunit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, DaemonApp, IniFiles,
  lnet, lnetbase, lcommon, Process, base;

type

  { TSKDaemon }

  TSKDaemon = class(TDaemon)
    procedure DataModuleExecute(Sender: TCustomDaemon);
    procedure DataModuleStart(Sender: TCustomDaemon; var OK: Boolean);
    procedure DataModuleStop(Sender: TCustomDaemon; var OK: Boolean);
  private
    TCP1    : TLTCP;
    ResetList : TStringList;
    procedure TCP1Accept(aSocket: TLSocket);
    procedure TCP1Receive(aSocket: TLSocket);
    procedure TCP1Error(const msg: string; aSocket: TLSocket);
    Procedure SKTimer;
    procedure LogMessage(S : String);
    Procedure TimerAdd(IP:String);
    Procedure TimerDel(Raum:String);
  public
    { public declarations }
  end;
Const
  Version = '0.28';


  msg_INOF   = 'INOF';    msg_INON   = 'INON';   // Internet On/Off
  msg_IROF   = 'IROF';    msg_IRON   = 'IRON';   // Intranet On/Off
  msg_UFOF   = 'UFOF';    msg_UFON   = 'UFON';   // URL-Filter On/Off
  msg_DROF   = 'DROF';    msg_DRON   = 'DRON';   // Raum-Drucker On/Off

  msg_GRST   = 'GRST';   // Get Room Default Status im akt. Raum
  msg_SRST   = 'SRST';   // Set Room Default
  msg_GROO   = 'GROO';   // Get alle PCs in akt. Raum: PC-Name,IP,MAC
  msg_GDRU   = 'GDRU';   // Get alle Drucker des Raums
  msg_GIPS   = 'GIPS';   // Get alle IPs im akt. Raum
  msg_AUTH   = 'AUTH';   // Credentials überprüfen: OK|DENY
  msg_SWOL   = 'SWOL';   // Wake On Lan für akt. Raum

  msg_GDST   = 'GDST';   // Get Drucker Status - Erster Drucker des Raums: ACCEPT|REJECT
  msg_GLIN   = 'GLIN';   // Get Logins der User im Raum über smbstatus

  msg_AARA   = 'AARA';   // Austeilen im aktuellen Raum
  msg_EARA   = 'EARA';   // Einsammeln und löschen
  msg_EARC   = 'EARC';   // Einsammeln Copie

  DefSKPort  = 9731;

var
  SKDaemon: TSKDaemon;

implementation

procedure RegisterDaemon;
begin
  RegisterDaemonClass(TSKDaemon)
end;



{$R *.lfm}

{ TSKDaemon }

procedure TSKDaemon.DataModuleStart(Sender: TCustomDaemon; var OK: Boolean);
Var
  IniFile        : TIniFile;
begin
  UseLog         := True;
  LogPfad        := DefLog_Pfad;
  LogMessage('########## Schukodaemon Version '+Version+' started ##########');
  OK := True;
  TCP1 := TLTCP.create(nil);
  TCP1.ReuseAddress := True;
  TCP1.Port    := DefSKPort;
  TCP1.Timeout := 100;
  TCP1.SocketNet := LAF_INET ;
  TCP1.OnReceive := @TCP1Receive;
  TCP1.OnAccept  := @TCP1Accept;
  TCP1.OnError   := @TCP1Error;
  SKPort         := DefSKPort;
  UseSSL         := false;
  debug          := false;
  useAuth        := 1;           // 0 - keine Authentififierung, 1 - mit Passwort , 2 - über host
  ResetTime      := 45;
  ResetList      := TStringList.create;
  if FileExists(Config_Pfad) then
    Begin
      IniFile := TIniFile.Create(Config_Pfad);
      Try with IniFile do
        Begin
          SKPort  := ReadInteger(Programm_sec,Port_entry,DefSKPort);
          UseLog  := Boolean(ReadInteger(Programm_sec,useLog_entry,1));
          UseSSL  := Boolean(ReadInteger(Programm_sec,useSSL_entry,0));
          debug   := Boolean(ReadInteger(Programm_sec,debug_entry,0));
          LogPfad := ReadString(Programm_sec,LogPfad_entry,DefLog_Pfad);
          UseAuth := ReadInteger(Programm_sec,useAuth_entry,1);
          ResetTime := ReadInteger(Programm_sec,ResetTime_entry,45);

          LogMessage('Konfigurationsdatei '+Config_Pfad+' geladen.');
        end
      finally
        IniFile.free;
      end;
    end
  else
    Begin
      LogMessage('Konfigurationsdatei nicht gefunden: '+Config_Pfad);
    end;
end;

procedure TSKDaemon.DataModuleStop(Sender: TCustomDaemon; var OK: Boolean);
begin
  OK := True;
  TCP1.Disconnect(True);
  Sleep(200);
  TCP1.free;
  ResetList.free;
end;

procedure TSKDaemon.DataModuleExecute(Sender: TCustomDaemon);
Var
  Time : integer;
begin
  if TCP1.Listen(SKPort) then LogMessage('Listen (Port: '+IntToStr(SKPort)+') -> true');
  repeat
    TCP1.CallAction;
    Sleep(200);
    Inc(Time);
    if (Time mod 200)=0 then SKTimer;
  until false;
end;

procedure TSKDaemon.TCP1Accept(aSocket: TLSocket);
Begin
  LogMessage('Accept: '+aSocket.PeerAddress);
end;

procedure TSKDaemon.TCP1Error(const msg: string; aSocket: TLSocket);
Begin
  LogMessage('ERROR: '+msg);
end;

procedure TSKDaemon.TCP1Receive(aSocket: TLSocket);
Var
  s,t,u,IP,Befehl : String;
  teacher,passwd  : String;
  msg             : TStringList;
  authOK          : Boolean;
Begin
  if aSocket.GetMessage(S) > 0 then
  Begin
    s := Trim(s);
    IP := aSocket.PeerAddress;
    LogMessage('Receive: '+Copy(S,1,4)+' from '+IP);
     if pos(msg_GIPS,s)=1 then
        Begin
          t := GetRaumIPs(IP);
          aSocket.SendMessage(t);
          exit;
        end;
     if pos(msg_GDST,s)=1 then
        Begin
          t := GetRaumDruckerStatus(IP);
          if t<>'' then aSocket.SendMessage(t)
                   else aSocket.SendMessage('KEINE');
          exit;
        end;
     msg := TStringList.create;
     Split('%',s,msg);
     if msg.count =3 then
        Begin
           if useAuth=0 then authOK := true
             else
               Begin;
                 authOK := false;
                 if (useAuth=2) and (CheckTeacherHost(msg[1],IP)) then authOK := True;
                 if (not authOK) then
                   Begin
                     LogMessage('... überprüfe Passwort ...');
                     if (CheckTeacherPW(msg[1],msg[2])) then authOK := True;
                   end;
               end;
           if authOK then
             Begin
               if msg[0]=msg_INOF then
                 Begin
                   Befehl := IN_script_Pfad+' --trigger=off --hostlist='+GetRaumIPs(IP);
                   Befehl_exec(Befehl);
                   LogMessage('Internet gesperrt ...');
                   TimerAdd(IP);
                 end;
               if msg[0]=msg_INON then
                 Begin
                   Befehl := IN_script_Pfad+' --trigger=on --hostlist='+GetRaumIPs(IP);
                   Befehl_exec(Befehl);
                   LogMessage('Internet freigegeben ...');
                   TimerAdd(IP);
                 end;
               if msg[0]=msg_DROF then
                 Begin
                   t := GetRaumDrucker(IP);
                   if t<>'' then
                     Begin
                       Befehl := 'cupsreject '+t;
                       Befehl_exec(Befehl);
                       LogMessage('Drucker gesperrt: '+t);
                     end;
                 end;
               if msg[0]=msg_DRON then
                 Begin
                   t := GetRaumDrucker(IP);
                   if t<>'' then
                     Begin
                       Befehl := 'cupsaccept '+t;
                       Befehl_exec(Befehl);
                       LogMessage('Drucker freigegeben: '+t);
                     end;
                 end;
               if msg[0]=msg_IROF then
                  Begin
                    Befehl := IR_script_Pfad+' --trigger=off --hostlist='+GetRaumIPs(IP);
                    Befehl_exec(Befehl);
                    LogMessage('Intranet gesperrt ...');
                    TimerAdd(IP);
                   end;
               if msg[0]=msg_IRON then
                  Begin
                    Befehl := IR_script_Pfad+' --trigger=on --hostlist='+GetRaumIPs(IP);
                    Befehl_exec(Befehl);
                    LogMessage('Intranet freigegeben ...');
                    TimerAdd(IP);
                  end;
               if msg[0]=msg_UFOF then
                  Begin
                    Befehl := UF_script_Pfad+' --trigger=off --hostlist='+GetRaumIPs(IP);
                    Befehl_exec(Befehl);
                    LogMessage('URL Filter nicht aktiv ...');
                    TimerAdd(IP);
                   end;
                if msg[0]=msg_UFON then
                  Begin
                    Befehl := UF_script_Pfad+' --trigger=on --hostlist='+GetRaumIPs(IP);
                    Befehl_exec(Befehl);
                    LogMessage('URL Filter aktiv ...');
                    TimerAdd(IP);
                  end;
               if msg[0]=msg_Auth then
                 Begin
                   aSocket.SendMessage('OK');
                 end;
               if msg[0]=msg_GRST then
                  Begin
                    t := GetRaum(IP);
                    if t<>'' then
                      Begin
                        u := GetRaumStatus(IP);
                        if u <>'' then aSocket.SendMessage(t+','+u)
                                  else aSocket.SendMessage('DENY');
                      end
                    else aSocket.SendMessage('DENY');
                  end;
               if msg[0]=msg_SRST then
                 Begin
                   t := GetRaum(IP);
                   if t <>'' then
                     Begin
                       Befehl := 'linuxmuster-reset --room='+t;
                       Befehl_exec(Befehl);
                       if LoescheRoomSession(t) then LogMessage('room_sessions gelöscht ... ');
                       LogMessage('Raum '+t+' auf Default-Werte gesetzt ...');
                       u := GetRaumStatus(IP);
                       if u <>'' then aSocket.SendMessage(t+','+u)
                                 else aSocket.SendMessage('DENY');
                       TimerDel(t);
                     end
                   else aSocket.SendMessage('DENY');
                 end;
               if msg[0]=msg_GLIN then
                 Begin
                   t := GetUserLogins(IP);
                   if t <>'' then aSocket.SendMessage(t)
                             else aSocket.SendMessage('DENY');
                   LogMessage('User-Logins: '+t);
                 end;
               if msg[0]=msg_GROO then
                 Begin
                   t := GetRaumPCInfos(IP);
                   if t <>'' then aSocket.SendMessage(t)
                             else aSocket.SendMessage('DENY');
                   LogMessage('Room-PCs: '+t);
                 end;
               if msg[0]=msg_GDRU then
                 Begin
                   t := GetRaumDrucker(IP);
                   if t <>'' then aSocket.SendMessage(t)
                             else aSocket.SendMessage('KEINE');
                 end;
               if msg[0]=msg_AARA then
                 Begin
                   t := GetUsersAktRaum(IP);
                   Befehl := 'sophomorix-teacher -t '+msg[1]+' --currentroom --handoutcopy --users '+t;
                   if t <>'' then Befehl_exec(Befehl);
                   LogMessage('Dateien im aktuellen Raum ausgeteilt an: '+t);
                 end;
               if msg[0]=msg_EARA then
                 Begin
                   t := GetUsersAktRaum(IP);
                   Befehl := 'sophomorix-teacher -t '+msg[1]+' --fromroom --collect --users '+t;
                   if t <>'' then Befehl_exec(Befehl);
                   LogMessage('Dateien im aktuellen Raum eingesammelt von: '+t);
                 end;
               if msg[0]=msg_EARC then
                 Begin
                   t := GetUsersAktRaum(IP);
                   Befehl := 'sophomorix-teacher -t '+msg[1]+' --fromroom --collectcopy --users '+t;
                   if t <>'' then Befehl_exec(Befehl);
                   LogMessage('Dateien im aktuellen Raum eingesammelt von: '+t);
                 end;
             end
           else
             Begin
               if msg[0]=msg_Auth then aSocket.SendMessage('DENY');
             end;
       end;
     msg.free;
   end;
end;

Procedure TSKDaemon.TimerAdd(IP:String);
Var
  s,R : String;
Begin
  R := GetRaum(IP);
  TimerDel(R);
  ResetList.Append(DateTimeToStr(now)+';'+R);
  LogMessage('Timer-Gesetzt-Raum: ' + R) ;
end;

Procedure TSKDaemon.TimerDel(Raum:String);
Var
  n,p : Integer;
  s,R : String;
Begin
  n :=0;
  While n<=ResetList.count-1 do
    Begin
      s := ResetList[n];
      p := pos(';',S);
      R := Copy(S,p+1,Length(s)-p);
      if R=Raum then
        Begin
          LogMessage('Timer-Gelöscht-Raum: ' + R) ;
          ResetList.Delete(n);
        end
      else
        inc(n);
    end;
end;


Procedure TSKDaemon.SKTimer;
Var
  n,p : Integer;
  s,R,Befehl : String;
Begin
  //LogMessage('Timer-Liste: '+ResetList.commatext);
  n :=0;
  While n<=ResetList.count-1 do
    Begin
      s := ResetList[n];
      p := pos(';',S);
      if StrToDateTime(Copy(s,1,p-1))+(ResetTime/(24*60))<now then
        Begin
          R := Copy(S,p+1,Length(s)-p);
          LogMessage('Timer-Reset-Raum: ' + R) ;
          Befehl := 'linuxmuster-reset --room='+R;
          Befehl_exec(Befehl);
          ResetList.Delete(n);
        end
      else
        inc(n);
    end;
  LastIP :='';
  LastRaum :='';
end;

procedure TSKDaemon.LogMessage(S : String);
Var
  Datei : TextFile;
  t     : String;
Begin
  t := FormatDateTime('dd.mm.yyyy-hh:nn:ss',now);
  Try
  if FileExists(LogPfad) then
    Begin
      AssignFile(Datei,LogPfad);
      Append(Datei);
      Writeln(Datei,t+' '+S);
      CloseFile(Datei);
    end
  else
    Begin
      AssignFile(Datei,LogPfad);
      Rewrite(Datei);
      Writeln(Datei,t+' '+S);
      CloseFile(Datei);
    end;
  except
    on E: EInOutError do
     writeln('File handling error occurred. Details: ', E.Message);
  end;
end;

initialization
  RegisterDaemon;
end.

